/* src/outils.c - Divers outils
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs_and_davidounet@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <time.h>
#include <stdarg.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#include <main.h>
#ifdef HAVE_CRYPTHOST
#include <crypt.h>
#endif

CVSINFO("$Id: outils.c,v 1.4 2003/02/16 01:15:25 romexzf Exp $");

int dataS = 0;
extern struct Link *mainhub;

static const char base64[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]";

extern struct badchans *badchead;
extern struct badnicks *badnhead;

extern struct cs_commands cs_cmds[];

static const char *months[12] = {
  "Janvier", "Fevrier", "Mars", "Avril",
  "Mai", "Juin", "Juillet", "Aot",
  "Septembre", "Octobre", "Novembre", "Decembre"
};

static const char *weekdays[7] = {
  "Dimanche", "Lundi", "Mardi", "Mercredi",
  "Jeudi", "Vendredi", "Samedi"
};

#ifdef HAVE_CRYPTHOST
char *hostprot(unsigned char *key, unsigned char *host)
{
  u_int32_t sum;
  u_int32_t digest[4];
  char *newhost, tmp[5], ID[64], *ptr;

  if(!host[0]%2)
  { // md2
     MD2_CTX context;
     MD2Init(&context);
     MD2Update(&context,key,strlen(key));
     MD2Update(&context,host,strlen(host));
     MD2Final((void *) digest,&context);
  }
  else
  { // md5
     MD5_CTX context;
     MD5Init(&context);
     MD5Update(&context,key,strlen(key));
     MD5Update(&context,host,strlen(host));
     MD5Final((void *) digest,&context);
  }

  sum=digest[0]+digest[1]+digest[2]+digest[3];

  sprintf(ID,"%X", sum);
  newhost = malloc(500);
  newhost[0] = '\0';
  if (!is_ip(host))
  {
    ptr=(char *) strchr(host, '.');
    strncpy(newhost, ID,256);
    strncat(newhost, ptr, HOSTLEN);
  } else
  {
    ptr=(char *) strrchr(host, '.');
    strcpy(tmp, host);
    strtok(tmp, ".");
    strncpy(newhost, tmp,5);
    strcat(newhost, ".");
    strncat(newhost, ID, 20);
    newhost[HOSTLEN]=0;
  }
  return(newhost);
}
#endif

/* Note: La fonction chan_check a t reprise du CS5.0  IRCore*/
int chan_check(char *chan, aNick *nick)
{
   struct badchans *badchan;
   char *p = chan;
   if(!chan || *p == '\0')
   {
      csntc(nick, "\002Veuillez rentrer un nom de salon\002");
      return 0;
   }

   if(strstr(chan, " "))
   {
      csntc(nick, "\002Le nom du salon ne doit pas comporter d'espaces\002");
      return 0;
   }
   if(strlen(chan) > 100)
   {
      csntc(nick, "\002Le nom du salon est trop long (limite 100)\002");
      return 0;
   }
   if(strchr(chan, ',') || *p != '#')
   {
      csntc(nick, "[\002%s\002] n'est pas un nom de salon correct", chan);
      return 0;
   }
   if(!strcasecmp(cs.chan,chan))
   {
	   csntc(nick, "Salon non enregistrable.");
	   return 0;
   }
   while(*p != '\0')
   {
      if(!isascii(*p))
      {
         csntc(nick, "\002Le nom du salon doit contenir aucun caractre spcial");
         return 0;
      }
      p++;
   }
   if((badchan = getbadchan(chan)))
   {
	   csntc(nick, "Le salon \002%s\002 ne peut pas tre enregistr pour la raison suivante : \002%s\002", chan, badchan->raison);
	   return 0;
   }
   return 1;
}

int is_num(char *num)
{
   while(*num)
   {
     if(!isdigit(*num)) return 0;
     num++;
   }
   return 1;
}

int is_ip(char *addr)
{
   while(*addr)
   {
     if(!isdigit(*addr)&&( *addr != '.' )) return 0;
     addr++;
   }
   return 1;
}

void strupr(char *buf)
{
   int i = 0;
   while(buf[i] != '\0')
   {
      buf[i] = toupper(buf[i]);
      i++;
   }
   buf[i] = '\0';
}

void strlwr(char *buf)
{
   int i = 0;
   while(buf[i] != '\0')
   {
      buf[i] = tolower(buf[i]);
      i++;
   }
   buf[i] = '\0';
}

void conv_char(char *chan, char *string)
{
	char val[10], *ptr = chan;

	val[0]='\0';
	strcpy(string, "");

	while(*ptr != '\0')
	{
		if(!isalnum(*ptr)) sprintf(val, "%%%03o", *ptr);
		else sprintf(val, "%c", *ptr);
		strcat(string, val);
		ptr++;
	}
	strcat(string, "\0");
	return;
}

void strip_newline(char *string)
{
   char *ptr = NULL;
   if((ptr = strchr(string, '\n'))) *ptr = '\0';
   if((ptr = strchr(string, '\r'))) *ptr = '\0';
}

int ChanLevelbyUserI(anUser *user, char *chan)
{
	anAccess *acces=NULL;
	if((acces=GetAccessIbyUserI(user,chan))) return acces->level;
	return 0;
}

struct joininfo *getjoininfo(struct nickinfo *nick, const char *chan)
{
	aJoin *join;
	for(join = nick->joinhead;join && strcasecmp(chan, join->channel);join=join->next);
	return join;
}

struct chaninfo *getchaninfo(const char *chan)
{
   aChan *current;
   for(current = chanhead;current && strcasecmp(chan,current->chan);current=current->next);
   return current;

}

struct userinfo *getuserinfo(const char *nick)
{
   struct userinfo *current;
   for(current = userhead;current && strcasecmp(nick,current->nick);current=current->next);
   return current;
}

struct nickinfo *getnickbynick(const char *nick)
{
   struct nickinfo *current;
   current = nickhead;
   while(current)
   {
      if(!strcasecmp(nick,current->nick)) return current;
      current = current->next;
   }
   return NULL;
}

struct badnicks *getbadnick(const char *chan)
{
   struct badnicks *current;
   for(current = badnhead;current;current=current->next) if(!match(current->chaine, chan)) return current;
   return NULL;
}

int IsBadNick(const char *chan)
{
   struct badnicks *current;
   for(current = badnhead;current;current=current->next) if(!match(current->chaine, chan)) return 1;
   return 0;
}

struct badchans *getbadchan(const char *chan)
{
   struct badchans *current;
   for(current=badchead;current && match(current->chaine, chan);current = current->next);
   return current;
}

char *IsOwner(char *nick, char *chan)
{
	anAccess *access;
	anUser *user;

	if(!(user=getuserinfo(nick))) return NULL;

	if((access=GetAccessIbyUserI(user,chan)) && access->level == 500) return access->chan;
	return NULL;
}

char *IsAnOwner(const char *nick)
{
	anAccess *access;
	anUser *user;

	user = userhead;
	while(user != NULL)
	{
		if(!strcasecmp(nick, user->nick))
		{

			access = user->accesshead;
			while(access != NULL)
			{
				if(access->level == 500) return access->chan;
				access = access->next;
			}
			return NULL;
		}
		user = user->next;
	}
	return NULL;
}

int IsRegChan(const char *chan)
{
   aChan *current;
   for(current = chanhead;current;current=current->next) if(!strcasecmp(chan,current->chan)) return 1;
   return 0;
}

int IsRegNick(char *nick)
{
   anUser *user;

   if((user=getuserinfo(nick))) return 1;
   return 0;
}

struct nickinfo *num2nickinfo(const char *num)
{
   struct nickinfo *nick;

   for(nick = nickhead;nick;nick=nick->next) if(!strcmp(num, nick->numeric)) return nick;
   return NULL;
}

char *num2nick(const char *num)
{
   struct nickinfo *nick;

   for(nick = nickhead;nick;nick=nick->next) if(!strcmp(num, nick->numeric)) return nick->nick;
   return NULL;
}

char *nick2num(const char *nick)
{
   struct nickinfo *current;

   for(current = nickhead;current;current=current->next) if(!strcasecmp(nick, current->nick)) return current->numeric;
   return NULL;
}

struct access *GetAccessIbyUserI(anUser *user, const char *chan)
{
	anAccess *a=NULL;
	for(a = user->accesshead ;a && strcasecmp(a->chan,chan) ; a = a->next);
	return a;
}

/*
 * getoption:
 * recherche dans un tableau char ** (de 'parc' entres)
 * l'expression "opt"  partir de l'lment 'deb'
 * -si exp_int est > 0:
 *  cela veut dire qu'on attend une valeur numerique apres le char 'opt'
 *  si c'est le cas, la fct renvoit la valeur, sinon 0
 * -si exp_int est < 0:
 *  cela veut dire que l'on recherche uniquement si 'opt' est dans parv,
 *  si c'est le cas, retourne 1
 * -si exp_int est = 0:
 *  cela veut dire que l'on attend une string apres le 'opt',
 *  si c'est le cas retourne l'index de cette string dans parv, sinon 0.
 *
 */

int getoption(char *opt, char **parv, int parc, int deb, int exp_int)
{
	for(;deb<=parc;deb++)
	{
		if(!strcasecmp(opt,parv[deb]))
		{
			if(exp_int < 0) return 1;
			if(++deb<=parc)
			{
				if(is_num(parv[deb]) && exp_int) return atoi(parv[deb]);
				else if(!exp_int) return deb;
			}
			break;
		}
	}
	return 0;
}

struct Link *num2servinfo(char *num)
{
	struct Link *link;

	for(link=mainhub;link;link=link->next)
	{
#ifdef N2K
		if(!strncmp(num,link->num,2)) return link;
#else
		if(!strncmp(num,link->num,1)) return link;
#endif
	}
	return NULL;
}

struct Link *GetLinkIbyServ(const char *serv)
{
	struct Link *link;
	for(link=mainhub;link && strcasecmp(link->serv,serv);link=link->next);
	return link;
}

static void log(const char *filename, const char *format, ...)
{
   va_list arglist;
   FILE *fp;
   if(!(fp = fopen(filename, "a"))) return;
   va_start(arglist, format);
   vfprintf(fp, format, arglist);
   va_end(arglist);
   fclose(fp);
}

void parv2msgn(unsigned int parc, char **parv, unsigned int base, char *buf, unsigned int size)
{
	unsigned int i, taillec, towrite;

	strcpy(buf,"");
	size--;
	for(i=base;i<=parc;i++)
	{
		if(!size) return;
		taillec=strlen(parv[i]);
		if(i>base)
		{
			strcat(buf, " ");
			size--;
		}
		if(size < taillec)
		{
			towrite=size;
			size=0;
		}
		else
		{
			towrite = taillec;
			size -= taillec;
		}
		strncat(buf, parv[i], towrite);
	}
	strcat(buf, "\0");
	return;
}

int ToggleFlag(int flags, int flag)
{
	if(flags & flag) flags &= ~flag;
	else flags |= flag;
	return flags;
}

void duration(int i, char *dur)
{
	int mins, hours, days;
	char tmp2[30];

	if(i >= 60)
	{
		mins = i / 60;
		i -= 60 * mins;
	}
	else mins = 0;
	if(mins >= 60)
	{
		hours = mins / 60;
		mins -= 60 * hours;
	}
	else hours = 0;
	if(hours >= 24)
	{
		days = hours / 24;
		hours -= 24 * days;
	}
	else days = 0;
	strcpy(dur, "");

	if(days) {
	  sprintf(tmp2, "\002%d\002 jours", days);
	  strcat(dur, tmp2);
	}
	if(hours) {
	  if(days) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\002 heures", hours);
	  strcat(dur, tmp2);
	}
	if(mins) {
	  if(days||hours) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\002 minutes", mins);
	  strcat(dur, tmp2);
	}
	if(!((mins || hours || days) && !i)) {
	  if(days||hours||mins) strcat(dur, " et ");
	  sprintf(tmp2, "\002%d\002 secondes", i);
	  strcat(dur, tmp2);
	}
	strcat(dur, "\0");
	return;
}

void get_time(long ttime, char *time)
{
  register struct tm *lt;
  time_t clock;

  clock = ttime;
  lt = localtime(&clock);

  snprintf(time, 70, "%s %d %s %d %02d:%02d:%02d",
      weekdays[lt->tm_wday], lt->tm_mday, months[lt->tm_mon],
      1900 + lt->tm_year, lt->tm_hour, lt->tm_min, lt->tm_sec);

  return;

}

char *bottime()
{
   time_t timer;
   struct tm *ntime;
   char *buf;
   buf = malloc(70);

   time(&timer);
   ntime=localtime(&timer);
   snprintf(buf, 70,"%02d:%02d:%02d",ntime->tm_hour,ntime->tm_min,ntime->tm_sec);
   return buf;
}

void putlog(char *fichier, const char *pattern, ...)
{
  char buf[512], t[70];
  va_list vl;

  if(fichier == 0 || fichier == '\0') return;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  get_time(time(NULL), t)
;  log(fichier, "[%s] %s", t, buf);
}

void putserv(const char *pattern, ...)
{
	char buf[700];
	int len;
#ifdef DEBUG
	char tmp[10], tmp2[10];
#endif
	va_list vl;
	buf[0]='\0';
	va_start(vl, pattern);
	vsnprintf(buf, 700, pattern, vl);
	va_end(vl);
	len = strlen(buf);
	send(bot.sock, buf, len,0);
	dataS += len;
#ifdef DEBUG
	snprintf(tmp, 10, "%s %s", bot.servnum, TOKEN_PONG);
	snprintf(tmp2, 10, "%s %s", bot.servnum, TOKEN_PING);
#ifdef N2K
	if(strncmp(buf, tmp, 4) && strncmp(buf, tmp2, 4))
#else
	if(strncmp(buf, tmp, 5) && strncmp(buf, tmp2, 5))
#endif
    putlog(LOG_PARSES, "S - %s",buf);
#endif
}

void csntc(aNick *nick, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv("%s %s %s :%s\n", cs.num, TOKEN_NOTICE, nick->numeric, buf);
  tidle = time(NULL);
}

void putchan(const char *buf)
{
  char *t;
  t=bottime();
  putserv("%s %s %s :[%s] %s\n", cs.num, TOKEN_PRIVMSG, cs.chan, t, buf);
  free(t);
  return;
}

void csmode(aChan *chan, const char *pattern, ...)
{
  char buf[512], *parv[100], modes[100], *limit = NULL, *key = NULL, *tmp = NULL, tmp2[5];
  int i=-1, what = 1;

  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv("%s %s %s %s\n", cs.num, TOKEN_MODE, chan, buf);

  modes[0]='\0';
  strtok(buf, " ");
  tmp=strtok(NULL, " ");
  while(tmp)
  {
	parv[++i] = tmp;
	tmp = strtok(NULL, " ");
  }
  tmp = buf;
  parv[++i] = NULL;
  i=0;
  while (tmp && *tmp)
  {
  	sprintf(tmp2, "%c", *tmp);
	switch (*tmp)
	{
		case '+': strcat(modes, tmp2); what = 1; break;
		case '-': strcat(modes, tmp2); what = 0; break;
		case 'o':
		case 'v':
		case 'b':
			i++; break;
		case 'l': if(what && parv[i] && is_num(parv[i]))
			  {
			  	limit = parv[i];
			  	i++;
			  }
			  strcat(modes, tmp2);
			  break;
		case 'k': if(parv[i])
			  {
			  	  key = parv[i];
			  	  i++;
			  }
			  strcat(modes, tmp2);
			  break;
		case 'i':
		case 'm':
		case 'r':
		case 't':
		case 'p':
		case 's':
		case 'n':
		case 'O':
		case 'c':
		case 'q':
		case 'C':
			strcat(modes, tmp2); break;
	}
	tmp++;
  }
  parse_cmode(chan, modes, limit, key);
  return;
}

void cstopic(aChan *chan, const char *pattern, ...)
{
  char buf[TOPICLEN];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, TOPICLEN, pattern, vl);
  va_end(vl);
  putserv("%s %s %s :%s\n", cs.num, TOKEN_TOPIC, chan->chan, buf);
  strcpy(chan->topic, buf);
}

void cskick(const char *chan, const char *vict, const char *pattern, ...)
{
  char buf[512];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 512, pattern, vl);
  va_end(vl);
  putserv("%s %s %s %s :%s\n", cs.num, TOKEN_KICK, chan, vict, buf);
}

void cswallops(const char *pattern, ...)
{
  char buf[300];
  va_list vl;
  buf[0]='\0';
  va_start(vl, pattern);
  vsnprintf(buf, 300, pattern, vl);
  va_end(vl);
  putserv("%s %s :%s\n", cs.num, TOKEN_WALLOPS, buf);
}

void csjoin(aChan *chan)
{
	aLink *lp;
	aNick *n;
	aJoin *j;

	putserv("%s %s %s\n", cs.num, TOKEN_JOIN, chan->chan);
	for(n=nickhead;n;n=n->next)
		for(j=n->joinhead;j;j=j->next)
		{
			if(!(strcasecmp(chan->chan, j->channel)))
			{	 /* on a trouv le bon chan hh */
				lp = (aLink *) calloc(1, sizeof(aLink));
				lp->value.j = j;
				lp->next = chan->members;
				chan->members = lp;
				break;
			}
		}
}

void cspart(const char *chan, const char *pattern, ...)
{
	char buf[300];
	va_list vl;
	buf[0]='\0';
	va_start(vl, pattern);
	vsnprintf(buf, 300, pattern, vl);
	va_end(vl);
	putserv("%s %s %s :%s\n", cs.num, TOKEN_PART, chan, buf);
}

void cs_account(aNick *nick, const char *account)
{
#ifdef N2K
	putserv("%s %s %s %s\n", bot.servnum, TOKEN_ACCOUNT, nick->numeric, account ? account : "");
#else
	putserv(":%s %s %s %s\n", cs.nick, TOKEN_ACCOUNT, nick->numeric, account ? account : "");
#endif
	return;
}

#ifdef N2K
char *make_servnum(int c)
{
	static char res[3];
	res[0] = base64[c / 64];
	res[1] = base64[c % 64];
	res[2] = '\0';
	return res;
}
#else
char *make_servnum(int c)
{
  int i;
  static char res[3];

  memset(res, 0, 3);
  for (i=0; i>=0; c = c >> 6, i--)
  {
    res[i] = base64[c & 63];
  }
  return res;
}
#endif

int resolve(char *host)
{
  struct hostent *he;
  he=gethostbyname(host);
  if(he==NULL) return(inet_addr(host));
  return (*(int *)he->h_addr);
}


int check_protect(aNick *executant, aNick *execute, char *chan)
{

   anAccess *access1, *access2;

   if(!strcasecmp(executant->nick, execute->nick)) return 1;
   else if(HasMode(execute, 'p') && !executant->is_oper && IsAnAdmin(executant->user))
   {
   	csntc(executant, "L'user \002%s\002 est +p", execute->nick);
	return 0;
   }
   else if(execute->user == NULL) return 1;
   else if(IsAnAdmin(executant->user) && executant->user->level > execute->user->level) return 1; /*l'executant est un admin de level suprieur a l'execut*/
  else if(IsAnAdmin(executant->user) && IsAnAdmin(execute->user) && executant->user->level <= execute->user->level)
   {
      csntc(executant, "L'user \002%s\002 est un Admin de niveau suprieur ou gal au votre.", execute->nick);
	  return 0; /*ce sont tous les 2 des admins mais executant est inferieur a execut*/
   }
   else if(!IsAnAdmin(executant->user) && IsAnAdmin(execute->user))
   {
      csntc(executant, "L'user \002%s\002 est un Administrateur des Services.", execute->nick);
	  return 0; /*executant n'est pas admin mais execute est admin*/
   }

    /*optimisation: on check dabord l'acces de l'execut!*/
   if(!(access2=GetAccessIbyUserI(execute->user, chan))) return 1; /*execut = sans acces*/
   if(!access2->flag & A_PROTECT) return 1; /*ca vaut pas le coup de boucler sil est pas protect..*/

   /*on boucle les access de l'executant pour voir si on en trouve 1 sur le chan*/
   if(!(access1=GetAccessIbyUserI(executant->user, chan))) return 0; /*ne doit jamais arriver, voudrait dire que executant n'est pas un admin et n'a pas d'access sur le chan*/

   if(access1->level > access2->level) return 1; /*on verifie si executant a un level superieur a execute, ou si execute n'est pas protect;*/

   csntc(executant, "L'user \002%s\002 est protg ou a un access suprieur au votre sur \002%s\002.", execute->nick, chan);
   return 0; /*arrive pour tous les autres cas*/

}

char *strip_lastword(char *rest)
{
   char temp[500], *tmp2 = NULL, *tmp3, *strbuff;
   int nbmots = 1, i = 1;

   strbuff = malloc(500);
   strbuff[0] ='\0';
   temp[0] ='\0';
   strncpy(temp, rest, 500);
   strcpy(tmp2,rest);
   while (*tmp2 != '\0')
   {
      if(*tmp2 == ' ') nbmots++;
      if(!strchr(tmp2, ' ')) break;
	  tmp2++;
   }
   if(nbmots==1) strncat(strbuff, "********", 500);
   else
   {
       tmp3 = strtok(temp, " ");
       strncpy(strbuff, tmp3, 500);
       i++;
   }
   while(i < nbmots)
   {
      tmp3 = strtok(NULL, " ");
	  snprintf(strbuff, 500, "%s %s", strbuff, tmp3);
	  i++;
   }
   if(nbmots != 1) snprintf(strbuff, 500, "%s ********", strbuff);
   strbuff[strlen(strbuff) + 1] ='\0';
   return strbuff;
}


int getgloballevel(aNick *nick)
{
   return !nick->user ? 0 : nick->user->level;
}

void parse_umode(aNick *nick, char *p)
{
	char add[15],del[15],cur[15],mode[10];
	int w=0,j;
	char *x=nick->modes;
	char tmp[2];
	add[0]='\0';
	del[0]='\0';
	tmp[0]='\0';
	cur[0]='\0';
	while(*p)
	{
		if(*p=='+') w = 1;
		else if(*p=='-') w = 0;
		else
		{
			sprintf(tmp, "%c", *p);
			if(w) strcat(add,tmp);
			else strcat(del,tmp);
		}
		p++;
	}
	while(*x)
	{
		j=0;
		w=0;
		while(del[j]!='\0')
		{
			if(del[j]==*x)
			{
				w=1;
				break;
			}
			j++;
		}
		if(!w)
		{
			sprintf(tmp, "%c", *x);
			strcat(cur,tmp);
		}
		x++;
	}
	strcpy(mode, "");
	strcat(mode,cur);
	strcat(mode,add);
	strncpy(nick->modes,mode,15);
	nick->is_oper=strchr(mode,'o')?1:0;
}

void parse_cmode(aChan *chan, char *p, char *limit, char *key)
{
	char add[15],del[15],cur[15],mode[10], *tmp2=NULL;
	int w=1,j,delkey=0,dellimit=0;
	char *x=chan->modes;
	char tmp[2];
	add[0]='\0';
	del[0]='\0';
	tmp[0]='\0';
	cur[0]='\0';
	strtok(x, " "); // si chan->mode = "+ntl 12", jarter le 12 et garder que les modes :)
	tmp2=strtok(NULL, " ");
	if(tmp2 && !limit) limit=tmp2;
	while(*p)
	{
		if(*p=='+') w = 1;
		else if(*p=='-') w = 0;
		else if(*p!='o' && *p!='b' && *p!='v')
		{
			sprintf(tmp, "%c", *p);
			if(w) strcat(add,tmp);
			else strcat(del,tmp);
		}
		if(*p=='k') delkey = w ? 0 : 1;
		if(*p=='l') dellimit = w ? 0 : 1;
		p++;
	}
	while(*x)
	{
		j=0;
		w=0;
		while(del[j]!='\0')
		{
			if(del[j]==*x)
			{
				w=1;
				break;
			}
			j++;
		}
		j=0;
		while(add[j]!='\0')
		{
			if(add[j]==*x)
			{
				w=1;
				break;
			}
			j++;
		}
		if(!w)
		{
			sprintf(tmp, "%c", *x);
			strcat(cur,tmp);
		}
		x++;
	}
	strcpy(mode, "");
	strcat(mode,cur);
	strcat(mode,add);
	if(limit && !dellimit) { strcat(mode, " "); strcat(mode, limit); }
	strncpy(chan->modes,mode,20);
	if(delkey) chan->key[0]='\0';
	else if(key) strncpy(chan->key, key, 100);
}

int IsValidNick(char *p)
{
	if(*p=='-' || isdigit(*p)) return 0;
	while(*p!='\0')
	{
		if(!isalpha(*p) && !isdigit(*p) && *p != '\\' && *p != '-' && *p != '{' && *p != '}'
		     && *p != '[' && *p != ']' && *p != '_' && *p != '|' && *p != '`') return 0;
			p++;
	}
	return 1;
}

int IsValidMail(char *p)
{
	int fa=0,fd=0,ispoint=0;
	if(*p=='-' || *p=='_' || *p=='.' || isdigit(*p) || *p=='@') return 0;
	while(*p!='\0')
	{
		if(*p=='@')
		{
			if(!fa) fa=1;
			else return 0;
		}
		if(fa && *p!='.') ispoint=0;
		if(fa && *p=='.')
		{
			if(ispoint) return 0; /* le char d'avant etait un point, 2 points successifs dans l'url interdits */
			fd=1;
			ispoint=1;
		}
		if(!isalpha(*p) && !isdigit(*p) && *p != '.' && *p != '-' && *p != '_' && *p!='@') return 0;
			p++;
	}
	if(fa && fd) return 1;
	return 0;
}

int GetCmdIndexbyName(char *name)
{
	int i=0;
	for(;cs_cmds[i].cmd[0]!='\0' && strcasecmp(name,cs_cmds[i].realcmd);i++);
	return i;
}

/*char *parv2msg(unsigned int parc, char **parv, unsigned int base)
{
  unsigned int i;
  parvbuff[0] = '\0';

  for(i=base;i<=parc;i++)
  {
    if(i>base) strcat(parvbuff, " ");
    strncat(parvbuff, parv[i], 500);
  }
  return parvbuff;
}*/

/*int b2n (int b)
{
  int i;
  for (i=0; i<64; i++)
  {
    if (base64[i] == b) return i;
  }
  return -1;
}

int decode_num (const char *s)
{
  int i,c,poids,num;
  num=0;
  poids=1;
  for (i=strlen(s)-1; i >= 0; i--)
  {
    c = b2n(s[i]);
    if (c == -1) return -1;
    num += poids*c;
    poids=poids*64;
  }
  return num;
}*/
